from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel


class LogEntry(BaseModel):
    date: datetime
    domainId: int
    domainType: str
    id: int
    level: int
    message: str
    personId: int
    simulatePersonId: Optional[int] = None


class LoginPersonColor(BaseModel):
    key: str
    shade: int


class LoginPersonDomainAttributes(BaseModel):
    firstName: str
    guid: Optional[str] = None
    lastName: str


class LoginPerson(BaseModel):
    apiUrl: str
    color: Optional[LoginPersonColor] = None
    domainIdentifier: str
    frontendUrl: str
    imageUrl: Optional[str] = None
    initials: str
    title: str
    domainAttributes: LoginPersonDomainAttributes
    domainType: str
    icon: str
    infos: List[str]


class LoginStatistic(BaseModel):
    lastLogin: datetime
    person: LoginPerson
    totalLogins: int


class SecurityLevel(BaseModel):
    id: int
    name: str
    sortkey: int



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel  # type: ignore


class LicenseSettings(BaseModel):
    hide_licensetab: str


class Config(BaseModel):
    _current_config_file: Optional[str] = None
    accept_datasecurity: Optional[bool] = None
    access_control_allow_credentials: Optional[bool] = None
    access_control_allow_origins: Optional[str] = None
    admin_ids: Optional[List[str]] = None
    admin_mail: Optional[str] = None
    admin_message: Optional[str] = None
    aiAssistantAvailable: Optional[bool] = None
    ai_description_available_generation_count: Optional[int] = None
    ai_description_available_generation_tests: Optional[int] = None
    ai_description_generation_count: Optional[int] = None
    ai_description_generation_enabled: Optional[bool] = None
    ai_description_total_generation_count: Optional[int] = None
    allowaiassistant: Optional[bool] = None
    allowcheckin: Optional[str] = None
    allowedcals: Optional[str] = None
    allowedclients: Optional[str] = None
    allowedresources: Optional[str] = None
    allowedservices: Optional[str] = None
    allowedstations: Optional[str] = None
    allowedsyncconnections: Optional[str] = None
    allowedsyncjobs: Optional[str] = None
    alloweduser: Optional[str] = None
    allowfinance: Optional[str] = None
    allowldap: Optional[str] = None
    allowoptigemsync: Optional[str] = None
    allowsync: Optional[str] = None
    alpha_book_affiliate_id: Optional[str] = None
    alpha_book_enabled: Optional[bool] = None
    app_security_request: Optional[bool] = None
    authorized_persons: Optional[str] = None
    brand: Optional[str] = None
    build: Optional[str] = None
    ccli_auto_reporting_enabled: Optional[bool] = None
    ccli_last_token_refresh: Optional[str] = None
    ccli_refresh_token: Optional[str] = None
    chatServer: Optional[str] = None
    chrome_active: Optional[str] = None
    chrome_binary: Optional[str] = None
    churchcal_active: Optional[bool] = None
    churchcal_css: Optional[str] = None
    churchcal_entries_last_days: Optional[int] = None
    churchcal_firstdayinweek: Optional[int] = None
    churchcal_maincalname: Optional[str] = None
    churchcal_name: Optional[str] = None
    churchcal_name_default: Optional[str] = None
    churchcal_sortcode: Optional[int] = None
    churchchat_allow_event_chat: Optional[bool] = None
    churchchat_allow_group_chat: Optional[bool] = None
    churchchat_allow_person_chat: Optional[bool] = None
    churchchat_delete_event_chat_after_x_days: Optional[int] = None
    churchchat_invite_ct_event_chat: Optional[bool] = None
    churchchat_invite_ct_group_chat: Optional[bool] = None
    churchchat_name: Optional[str] = None
    churchchat_name_default: Optional[str] = None
    churchchat_sortcode: Optional[int] = None
    churchchat_start_event_chat_before_x_days: Optional[int] = None
    churchchat_start_event_chat_for_calendars: Optional[str] = None
    churchchat_sync_user_id: Optional[int] = None
    churchcheckin_active: Optional[bool] = None
    churchcheckin_label_child: Optional[str] = None
    churchcheckin_label_parent: Optional[str] = None
    churchcheckin_label_standard: Optional[str] = None
    churchcheckin_name: Optional[str] = None
    churchcheckin_name_default: Optional[str] = None
    churchcheckin_sortcode: Optional[int] = None
    churchcheckin_tags: Optional[str] = None
    churchcustommodule_active: Optional[str] = None
    churchcustommodule_name: Optional[str] = None
    churchcustommodule_name_default: Optional[str] = None
    churchdb_active: Optional[bool] = None
    churchdb_archivedeletehistory: Optional[bool] = None
    churchdb_birthdaylist_station: Optional[str] = None
    churchdb_birthdaylist_status: Optional[str] = None
    churchdb_cleverreach_client_id: Optional[str] = None
    churchdb_cleverreach_client_secret: Optional[str] = None
    churchdb_cleverreach_connected: Optional[bool] = None
    churchdb_emailseparator: Optional[str] = None
    churchdb_groupnotchoosable: Optional[int] = None
    churchdb_home_lat: Optional[str] = None
    churchdb_home_lng: Optional[str] = None
    churchdb_mailchimp_apikey: Optional[str] = None
    churchdb_mailchimp_connected: Optional[bool] = None
    churchdb_mailjet_apikey: Optional[str] = None
    churchdb_mailjet_apisecret: Optional[str] = None
    churchdb_mailjet_connected: Optional[bool] = None
    churchdb_memberlist_station: Optional[str] = None
    churchdb_memberlist_status: Optional[str] = None
    churchdb_name: Optional[str] = None
    churchdb_name_default: Optional[str] = None
    churchdb_sendgroupmails: Optional[bool] = None
    churchdb_smspromote_apikey: Optional[str] = None
    churchdb_sortcode: Optional[int] = None
    churchfinance_active: Optional[bool] = None
    churchfinance_name: Optional[str] = None
    churchfinance_name_default: Optional[str] = None
    churchfinance_sortcode: Optional[int] = None
    churchgroup_active: Optional[bool] = None
    churchgroup_inmenu: Optional[bool] = None
    churchgroup_name: Optional[str] = None
    churchgroup_name_default: Optional[str] = None
    churchgroup_sortcode: Optional[str] = None
    churchreport_active: Optional[bool] = None
    churchreport_name: Optional[str] = None
    churchreport_name_default: Optional[str] = None
    churchreport_sortcode: Optional[int] = None
    churchresource_active: Optional[bool] = None
    churchresource_anonymize_for_public_user: Optional[bool] = None
    churchresource_entries_last_days: Optional[int] = None
    churchresource_name: Optional[str] = None
    churchresource_name_default: Optional[str] = None
    churchresource_send_emails: Optional[bool] = None
    churchresource_sortcode: Optional[int] = None
    churchservice_active: Optional[bool] = None
    churchservice_agendashowenumeration: Optional[bool] = None
    churchservice_ccli_token: Optional[str] = None
    churchservice_ccli_token_secret: Optional[str] = None
    churchservice_entries_last_days: Optional[int] = None
    churchservice_invite_persons: Optional[bool] = None
    churchservice_name: Optional[str] = None
    churchservice_name_default: Optional[str] = None
    churchservice_openservice_rememberdays: Optional[int] = None
    churchservice_reminderhours: Optional[int] = None
    churchservice_songwithcategoryasdir: Optional[bool] = None
    churchservice_sortcode: Optional[int] = None
    churchsync_active: Optional[bool] = None
    churchsync_inmenu: Optional[bool] = None
    churchsync_name: Optional[str] = None
    churchsync_name_default: Optional[str] = None
    churchsync_sortcode: Optional[str] = None
    churchwiki_active: Optional[bool] = None
    churchwiki_name: Optional[str] = None
    churchwiki_name_default: Optional[str] = None
    churchwiki_sortcode: Optional[int] = None
    cron_daily: Optional[datetime] = None
    cron_hour_8: Optional[datetime] = None
    cronjob_dbdump: Optional[bool] = None
    cronjob_delay: Optional[int] = None
    csrf_enabled: Optional[bool] = None
    currently_mail_sending: Optional[str] = None
    datasecurity_banner_enabled: Optional[bool] = None
    datasecurity_privacy_agreement_text: Optional[str] = None
    datasecurity_privacy_agreement_text_for_children: Optional[str] = None
    db_name: Optional[str] = None
    db_password: Optional[str] = None
    db_server: Optional[str] = None
    db_user: Optional[str] = None
    default_phone_area_code: Optional[str] = None
    emailServer: Optional[str] = None
    encryptionkey: Optional[str] = None
    env: Optional[str] = None
    evangelische_termine_api_key: Optional[str] = None
    evangelische_termine_enabled: Optional[bool] = None
    evangelische_termine_name: Optional[str] = None
    evangelische_termine_url: Optional[str] = None
    evangelische_termine_vid: Optional[str] = None
    feature_custommodule: Optional[str] = None
    feature_darkmode: Optional[str] = None
    feature_group: Optional[str] = None
    feature_sync: Optional[str] = None
    finance_active: Optional[bool] = None
    finance_inmenu: Optional[str] = None
    finance_name: Optional[str] = None
    finance_name_default: Optional[str] = None
    finance_sortcode: Optional[str] = None
    first_sync_job: Optional[datetime] = None
    first_transaction: Optional[datetime] = None
    hostingservice: Optional[str] = None
    https_only: Optional[str] = None
    image_extension: Optional[str] = None
    installation_verification_code: Optional[str] = None
    invite_email_text: Optional[str] = None
    isPostsActive: Optional[bool] = None
    isSamlActive: Optional[bool] = None
    is_churchtools_blog_widget_active: Optional[bool] = None
    is_churchtools_onboarding_widget_active: Optional[bool] = None
    is_pr_widget_active: Optional[bool] = None
    is_rss_widget_active: Optional[bool] = None
    language: Optional[str] = None
    last_cron: Optional[str] = None
    last_cron_finished: Optional[str] = None
    last_db_dump: Optional[str] = None
    last_import_clear: Optional[str] = None
    last_translation_update: Optional[str] = None
    ldap_otp_enabled: Optional[bool] = None
    licenseSettings: Optional[LicenseSettings] = None
    log_debug: Optional[str] = None
    login_message: Optional[str] = None
    mail_enabled: Optional[bool] = None
    mail_sending_in_background: Optional[str] = None
    mail_sending_starttime: Optional[str] = None
    mail_smtp_args_host: Optional[str] = None
    mail_smtp_args_password: Optional[str] = None
    mail_smtp_args_port: Optional[str] = None
    mail_smtp_args_smtpsecure: Optional[str] = None
    mail_smtp_args_username: Optional[str] = None
    max_uploadfile_size_kb: Optional[int] = None
    memberlist_birthday_full: Optional[str] = None
    memberlist_email: Optional[str] = None
    memberlist_fax: Optional[str] = None
    memberlist_group_couples: Optional[str] = None
    memberlist_picture: Optional[str] = None
    memberlist_salutation: Optional[str] = None
    memberlist_telefongeschaeftlich: Optional[str] = None
    memberlist_telefonhandy: Optional[str] = None
    memberlist_telefonprivat: Optional[str] = None
    onboarding_start: Optional[datetime] = None
    openstreetmaps_enabled: Optional[bool] = None
    orderstatus: Optional[str] = None
    orderstatus_since_date: Optional[datetime] = None
    package: Optional[str] = None
    post_active: Optional[bool] = None
    post_edit_time_limited: Optional[bool] = None
    post_email_summary_default_enabled: Optional[bool] = None
    post_featured_groups: Optional[str] = None
    post_name: Optional[str] = None
    post_sortcode: Optional[int] = None
    post_wizard_completed: Optional[bool] = None
    post_wizard_groups: Optional[str] = None
    prevent_change_security_settings: Optional[bool] = None
    prevent_export: Optional[bool] = None
    privacy_policy_external: Optional[bool] = None
    privacy_policy_external_link: Optional[str] = None
    privacy_policy_fields_mandatory: Optional[bool] = None
    privacy_policy_fields_not_mandatory_api: Optional[bool] = None
    privacy_policy_internal: Optional[bool] = None
    privacy_policy_relationships: Optional[str] = None
    profile: Optional[str] = None
    rabbitmq_config_host: Optional[str] = None
    rabbitmq_config_password: Optional[str] = None
    rabbitmq_config_port: Optional[str] = None
    rabbitmq_config_user: Optional[str] = None
    rss_widget_link: Optional[str] = None
    safe_mode_enable_authorized_persons: Optional[str] = None
    safe_mode_enable_chat_sync: Optional[str] = None
    safe_mode_enable_consolidation: Optional[str] = None
    safe_mode_enable_guid_sync: Optional[str] = None
    safe_mode_enable_job_queueing: Optional[str] = None
    safe_mode_enable_mail: Optional[str] = None
    safe_mode_enable_newsletter: Optional[str] = None
    safe_mode_enable_notification: Optional[str] = None
    send_data_security_mails: Optional[bool] = None
    short_name: Optional[str] = None
    show_remember_me: Optional[bool] = None
    site_language: Optional[str] = None
    site_licensekey: Optional[str] = None
    site_logo: Optional[str] = None
    site_mail: Optional[str] = None
    site_name: Optional[str] = None
    site_offline: Optional[bool] = None
    site_startpage: Optional[str] = None
    site_url: Optional[str] = None
    support_user_active_since: Optional[datetime] = None
    test: Optional[str] = None
    timezone: Optional[str] = None
    version: Optional[str] = None
    webchatLink: Optional[str] = None
    website_order_status: Optional[str] = None
    website_sync_user_id: Optional[str] = None
    website_trial_user_id: Optional[int] = None
    website_url: Optional[str] = None
    welcome: Optional[str] = None
    welcome_subtext: Optional[str] = None


class SearchResult(BaseModel):
    apiUrl: str
    domainIdentifier: str
    domainType: str
    frontendUrl: str
    icon: str
    imageUrl: Optional[str] = None
    title: str

    # TODO: domainAttributes

    def __repr__(self) -> str:
        return f"<SearchResult: {self.title} [{self.domainType}]>"


class SimulateStop(BaseModel):
    redirect: str


class VersionInfo(BaseModel):
    build: str
    shortName: str
    siteName: str
    version: str

    def __repr__(self) -> str:
        return f"<Version: {self.version} (build: {self.build})>"



from datetime import date, datetime
from typing import List, Optional

from pydantic import BaseModel  # type: ignore

from . import EmptyStrToNone


class Person(BaseModel):
    id: Optional[int] = None
    securityLevelForPerson: Optional[int] = None
    editSecurityLevelForPerson: Optional[int] = None
    title: Optional[str] = None
    firstName: Optional[str] = None
    lastName: Optional[str] = None
    nickname: Optional[str] = None
    job: Optional[str] = None
    street: Optional[str] = None
    addressAddition: Optional[str] = None
    zip: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None
    latitude: Optional[EmptyStrToNone[float]] = None
    longitude: Optional[EmptyStrToNone[float]] = None
    latitudeLoose: Optional[EmptyStrToNone[float]] = None
    longitudeLoose: Optional[EmptyStrToNone[float]] = None
    phonePrivate: Optional[str] = None
    phoneWork: Optional[str] = None
    mobile: Optional[str] = None
    fax: Optional[str] = None
    birthName: Optional[str] = None
    birthday: Optional[EmptyStrToNone[date]] = None
    birthplace: Optional[str] = None
    imageUrl: Optional[str] = None
    familyImageUrl: Optional[str] = None
    sexId: Optional[int] = None
    email: Optional[str] = None
    cmsUserId: Optional[str] = None
    optigemUserId: Optional[str] = None
    nationalityId: Optional[int] = None
    familyStatusId: Optional[int] = None
    weddingDate: Optional[EmptyStrToNone[date]] = None
    campusId: Optional[int] = None
    statusId: Optional[int] = None
    departmentIds: Optional[List[int]] = None
    firstContact: Optional[EmptyStrToNone[datetime]] = None
    dateOfBelonging: Optional[EmptyStrToNone[date]] = None
    dateOfEntry: Optional[EmptyStrToNone[datetime]] = None
    dateOfResign: Optional[EmptyStrToNone[datetime]] = None
    dateOfBaptism: Optional[EmptyStrToNone[date]] = None
    placeOfBaptism: Optional[str] = None
    baptisedBy: Optional[str] = None
    referredBy: Optional[str] = None
    referredTo: Optional[str] = None
    growPathId: Optional[int] = None
    isArchived: Optional[bool] = None

    # TODO: privacyPolicyAgreement
    # TODO: meta

    def __repr__(self) -> str:
        return f"<Person: {self.firstName} {self.lastName} [{self.id}]>"


class PersonTag(BaseModel):
    id: int
    name: str
    count: int


class PersonDomainAttributes(BaseModel):
    firstName: str
    lastName: str
    guid: str


class PersonDomainObject(BaseModel):
    domainAttributes: PersonDomainAttributes
    imageUrl: str
    frontendUrl: str
    apiUrl: str
    domainType: str
    title: str
    domainIdentifier: str


class PersonRelationship(BaseModel):
    relative: PersonDomainObject
    degreeOfRelationship: str
    relationshipName: str
    relationshipTypeId: int

    def __repr__(self) -> str:
        return (
            f"<Relationship: {self.relative.title} " f"({self.degreeOfRelationship})>"
        )


class BirthdayPersonDomainAttributes(BaseModel):
    firstName: str
    lastName: str
    guid: str


class BirthdayPerson(BaseModel):
    title: str
    domainType: str
    domainIdentifier: str
    apiUrl: str
    frontendUrl: str
    imageUrl: Optional[str] = None
    domainAttributes: BirthdayPersonDomainAttributes

    def __repr__(self) -> str:
        return f"<BirthdayPerson: {self.title}>"


class Birthday(BaseModel):
    type: str
    date: date
    age: Optional[int] = None
    person: BirthdayPerson

    def __repr__(self) -> str:
        return f"<Birthday: {self.person.title} {self.date} " f"({self.age} years)>"


class Device(BaseModel):
    id: str
    type: str
    ttl: datetime
    version: str
    createdAt: datetime
    updatedAt: datetime



"""
Resource
========

CRUD methods for Resources

"""

from datetime import datetime
from typing import Any, Dict, List, Optional, Tuple

from churchtools.models import resource
from churchtools.models.resource import Booking, Resource, ResourceType


class Resources:
    def __init__(self, ct: Any) -> None:
        """Initialize a Resources object."""

        self.__ct = ct

    def bookings(
        self,
        resource_ids: List[int],
        status_ids: Optional[List[int]] = None,
        from_: Optional[datetime] = None,
        to: Optional[datetime] = None,
    ) -> List[Booking]:
        """Returns all departments"""

        params: Dict[str, Any] = {"resource_ids": resource_ids}

        if status_ids:
            params["status_ids"] = status_ids
        if from_:
            params["from_"] = f"{from_.year}-{from_.month:02d}-" f"{from_.day:02d}"
        if to:
            params["to"] = f"{to.year}-{to.month:02d}-" f"{to.day:02d}"

        res = self.__ct.make_request("bookings", params=params)

        bookings = []
        if res and "data" in res:
            for item in res["data"]:
                if "base" in item:
                    bookings.append(Booking(**item["base"]))

        return bookings

    def masterdata(self) -> Tuple[List[ResourceType], List[Resource]]:
        res = self.__ct.make_request("resource/masterdata")

        resource_types = []
        resources = []
        if res and "data" in res:
            if "resourceTypes" in res["data"]:
                for rtype in res["data"]["resourceTypes"]:
                    resource_types.append(ResourceType(**rtype))
            if "resources" in res["data"]:
                for resource in res["data"]["resources"]:
                    resources.append(Resource(**resource))

        return (
            resource_types,
            resources,
        )



import json
from datetime import datetime, timedelta
from typing import Any, Dict, List, Optional

from .models.calendar import Appointment, Calendar


class Calendars:
    def __init__(self, ct: Any) -> None:
        """Initialize an Calendar object."""

        self.__ct = ct

    def list(self) -> List[Calendar]:
        """Get all calendars

        :returns: A list of calendars
        :rtype: List[Calendar]
        """

        route = "calendars"
        res = self.__ct.make_request(route)

        calendars = []
        if "data" in res:
            for item in res["data"]:
                cal = Calendar(**item)
                calendars.append(cal)

        return calendars

    def appointments(
        self,
        calendar_ids: List[int],
        start_date: datetime = datetime.now(),
        end_date: datetime = datetime.now() + timedelta(weeks=4),
    ) -> List[Appointment]:
        """Get all appointments

        :returns: A list of appointments
        :rtype: List[Appointment]
        """

        params: Dict[str, Any] = {}

        params["calendar_ids"] = calendar_ids

        if start_date:
            params["from"] = (
                f"{start_date.year}-{start_date.month:02d}-" f"{start_date.day:02d}"
            )
        if end_date:
            params["to"] = (
                f"{end_date.year}-{end_date.month:02d}-" f"{end_date.day:02d}"
            )

        route = "calendars/appointments"
        res = self.__ct.make_request(route, params=params)

        apps = []
        if "data" in res:
            for item in res["data"]:
                if "base" in item:
                    item["base"]["startDate"] = item["calculated"]["startDate"]
                    item["base"]["endDate"] = item["calculated"]["endDate"]
                    try:
                        app = Appointment(**item["base"])
                        apps.append(app)
                    except Exception as e:
                        print(item)
                        print(e)

        return apps

    def create_appointment(
        self, calendar_id: int, appointment: Appointment
    ) -> Optional[Appointment]:
        """Create a new appointment

        .. note:: Dates have to be UTC!

        :param calendar_id: The ID of the calendar
        :type calendar_id: int
        :param appointment: The new appointment
        :type appointment: Appointment
        :returns: The newly created appointment
        :rtype: Appointment
        """

        route = f"calendars/{calendar_id}/appointments"
        res = self.__ct.make_request(
            route, method="post", data=json.loads(appointment.json())
        )

        if "data" in res:
            return Appointment(**res["data"])

        return None

    def get_appointment(
        self, calendar_id: int, appointment_id: int
    ) -> Optional[Appointment]:
        """Get an appointment by ID

        :param calendar_id: The ID of the calendar
        :type calendar_id: int
        :param appointment_id: The ID of the appointment
        :type appointment_id: int
        :returns: The Appointment
        :rtype: Appointment
        """

        route = f"calendars/{calendar_id}/appointments/{appointment_id}"
        res = self.__ct.make_request(route)

        if "data" in res:
            if "appointment" in res["data"]:
                return Appointment(**res["data"]["appointment"])

        return None

    def update_appointment(
        self, calendar_id: int, appointment_id: int, appointment: Appointment
    ) -> Optional[Appointment]:
        """Update an appointment

        .. note:: Dates have to be UTC!

        :param calendar_id: The ID of the calendar
        :type calendar_id: int
        :param appointment_id: The ID of the appointment
        :type appointment_id: int
        :param appointment: The updated appointment
        :type appointment: Appointment
        :returns: The updated appointment
        :rtype: Appointment
        """

        route = f"calendars/{calendar_id}/appointments/{appointment_id}"
        res = self.__ct.make_request(
            route, method="put", data=json.loads(appointment.json())
        )

        if "data" in res:
            return Appointment(**res["data"])

        return None

    def delete_appointment(self, calendar_id: int, appointment_id: int) -> bool:
        """Delete an appointment

        :param calendar_id: The ID of the calendar
        :type calendar_id: int
        :param appointment_id: The ID of the appointment
        :type appointment_id: int
        :param appointment: The updated appointment
        :type appointment: Appointment
        :returns: Success
        :rtype: bool
        """

        route = f"calendars/{calendar_id}/appointments/{appointment_id}"
        status_code = self.__ct.make_request(
            route, method="delete", return_status_code=True
        )
        return status_code == 204



from typing import Dict, List, Optional

from pydantic import BaseModel

from churchtools.models.group import AgeGroup
from churchtools.models.person import PersonDomainObject


class GroupHomepageDomainAttributes(BaseModel):
    childGroupIds: Optional[List[int]] = None


class GroupHomepageSimple(BaseModel):
    apiUrl: Optional[str] = None
    domainAttributes: Optional[GroupHomepageDomainAttributes] = None
    frontendUrl: Optional[str] = None
    icon: Optional[str] = None
    imageUrl: Optional[str] = None
    title: Optional[str] = None

    def get_hash(self) -> str:
        if self.apiUrl:
            return self.apiUrl.split("/")[-1]

        return ""


class GroupInformation(BaseModel):
    ageGroups: Optional[List[AgeGroup]] = None
    campus: Optional[Dict] = None
    groupCategory: Optional[Dict] = None
    groupPlaces: Optional[List[Dict]] = None
    imageUrl: Optional[str] = None
    leader: Optional[List[PersonDomainObject]] = None
    meetingTime: Optional[str] = None
    note: Optional[str] = None
    targetGroup: Optional[Dict] = None
    weekday: Optional[Dict] = None


class GroupDetail(BaseModel):
    allowWaitinglist: Optional[bool] = None
    autoAccept: Optional[bool] = None
    canSignUp: Optional[bool] = None
    children: Optional[List[int]] = None
    currentMemberCount: Optional[int] = None
    id: Optional[int] = None
    information: Optional[GroupInformation] = None
    maxMemberCount: Optional[int] = None
    name: Optional[str] = None
    requestedPlacesCount: Optional[int] = None
    requestedWaitinglistPlacesCount: Optional[int] = None
    signUpConditions: Optional[Dict] = None
    signUpHeadline: Optional[str] = None
    signUpPersons: Optional[Dict] = None


class GroupHomepage(BaseModel):
    defaultView: Optional[str] = None
    filter: Optional[List[Dict]] = None
    groups: List[GroupDetail]
    id: Optional[int] = None
    isEnabled: Optional[bool] = None
    meta: Optional[Dict] = None
    orderDirection: Optional[str] = None
    parentGroup: Optional[int] = None
    randomUrl: Optional[str] = None
    showFilter: Optional[bool] = None
    showGroups: Optional[bool] = None
    showLeader: Optional[bool] = None
    showMap: Optional[bool] = None
    sortBy: Optional[str] = None



from datetime import date, datetime
from typing import List, Optional, Union

from pydantic import BaseModel

from churchtools.models.calendar import AppointmentMeta, CalendarMeta


class BookingAdditional(BaseModel):
    date: Union[date, datetime]
    id: int
    isRepeated: Optional[bool] = None
    meta: CalendarMeta


class ResourceType(BaseModel):
    campusId: Optional[int] = None
    id: int
    name: str
    nameTranslated: str
    sortKey: int


class Resource(BaseModel):
    adminIds: Optional[List[int]] = None
    doesRequireCalEntry: bool
    iCalLocation: Optional[str] = None
    id: int
    isAutoAccept: bool
    isVirtual: bool
    location: Optional[str] = None
    name: str
    nameTranslated: str
    randomString: str
    resourceTypeId: int
    sortKey: int


class Booking(BaseModel):
    additionals: List[BookingAdditional]
    allDay: bool
    calId: Optional[int] = None
    caption: str
    endDate: Union[date, datetime]
    exceptions: List[BookingAdditional]
    id: int
    location: Optional[str] = None
    meta: Optional[AppointmentMeta] = None
    note: Optional[str] = None
    personId: Optional[int] = None
    repeatFrequency: Optional[int] = None
    repeatId: int
    repeatOption: Optional[int] = None
    repeatUntil: Optional[str] = None
    resource: Resource
    showInCal: bool
    startDate: Union[date, datetime]
    statusId: int
    version: int



"""
Groups
======

Groups we love

"""

from typing import Any, List

from .models.group import AgeGroup


class Groups:
    def __init__(self, ct: Any) -> None:
        self.__ct = ct

    def agegroups(self) -> List[AgeGroup]:
        """Get agegroups

        :returns: A list of all age groups
        :rtype: List[AgeGroup]
        """

        res = self.__ct.make_request("group/agegroups")

        agegroups = []
        if res and "data" in res:
            for item in res["data"]:
                agegroups.append(AgeGroup(**item))

        return agegroups



from typing import List, Optional

from pydantic import BaseModel  # type: ignore


class ArrangementLink(BaseModel):
    domainType: str
    domainId: str
    name: str
    filename: str
    fileUrl: str


class ArrangementFile(BaseModel):
    domainType: str
    domainId: str
    name: str
    filename: str
    fileUrl: str


class Arrangement(BaseModel):
    id: int
    name: str
    isDefault: bool
    keyOfArrangement: str
    bpm: str
    beat: str
    duration: int
    note: str
    links: List[ArrangementLink]
    files: List[ArrangementFile]

    # TODO: meta


class SongCategory(BaseModel):
    id: int
    name: str
    nameTranslated: str
    sortKey: int
    campusId: Optional[int] = None


class Song(BaseModel):
    id: int
    name: str
    category: SongCategory
    shouldPractice: bool
    author: str
    ccli: str
    copyright: str
    note: str
    arrangements: List[Arrangement]

    # TODO: meta

    def __repr__(self) -> str:
        return f"<Song: {self.name} [{self.id}]>"



"""
GroupHomepages
======

Groups we love

"""

from typing import Any, List

from .models.group_homepage import GroupHomepage, GroupHomepageSimple


class GroupHomepages:
    def __init__(self, ct: Any) -> None:
        self.__ct = ct

    def list(self) -> List[GroupHomepageSimple]:
        """Get all GroupHomepages

        :returns: A list of all group homepages
        :rtype: List[GroupHomepageSimple]
        """

        res = self.__ct.make_request("grouphomepages")

        homepages = []
        if res and "data" in res:
            for item in res["data"]:
                homepages.append(GroupHomepageSimple(**item))

        return homepages

    def get(self, hash: str) -> GroupHomepage:
        """Get a GroupHomepage by hash

        :returns: A GroupHomepage
        :rtype: GroupHomepage
        """

        res = self.__ct.make_request("grouphomepages/" + hash)

        return GroupHomepage(**res["data"])



"""
Songs
=====

Endpoints for Songs

"""

from typing import Any, Dict, List, Optional, Tuple

from .models.pagination import MetaPagination
from .models.song import Song


class Songs:
    def __init__(self, ct: Any) -> None:
        self.__ct = ct

    def list(
        self,
        song_category_ids: Optional[List[int]] = None,
        ids: Optional[List[int]] = None,
        practice: bool = False,
        key_of_arrangement: Optional[str] = None,
        page: int = 1,
        limit: int = 10,
    ) -> Tuple[List[Song], Optional[MetaPagination]]:
        """Return a list of songs.

        :param song_category_ids: Only query for these category IDs
        :type song_category_ids: List[int]
        :param ids: Only query for these IDs
        :type ids: List[int]
        :param practice: Query for songs that "should be practiced"
        :type practice: bool
        :param key_of_arrangement: Query for keys of an arrangement of the song
        :type key_of_arrangement: str
        :param page: Result page number
        :type page: int
        :param limit: Result item count
        :type limit: int
        :returns: A list of songs that match the query
        :rtype: List[Song]
        """

        params: Dict[str, Any] = {}

        if song_category_ids:
            params["song_category_ids"] = song_category_ids
        if ids:
            params["ids"] = ids
        if practice:
            params["practice"] = practice
        if key_of_arrangement:
            params["key_of_arrangement"] = key_of_arrangement
        if page:
            params["page"] = page
        if limit:
            params["limit"] = limit

        res = self.__ct.make_request("songs", params=params)

        songs = []
        if res and "data" in res:
            for item in res["data"]:
                song = Song(**item)
                songs.append(song)

        pagination = None
        if res and "meta" in res:
            pagination = MetaPagination(**res["meta"])

        return songs, pagination

    def get(self, song_id: int) -> Optional[Song]:
        """Get a song by id.

        :param song_id: The ID of the song
        :type song_id: int
        :returns: The song
        :rtype: Song
        """

        route = f"songs/{song_id}"
        res = self.__ct.make_request(route)

        if res and "data" in res:
            return Song(**res["data"])

        return None



from typing import List, Union

from pydantic import BaseModel


class Setting(BaseModel):
    module: str
    attribute: str
    value: Union[dict, int, List, str]

    def __repr__(self) -> str:
        return f"<Setting: {self.module}.{self.attribute} = {self.value}>"



from typing import Optional

from pydantic import BaseModel

from . import EmptyStrToFalse


class WikiCategory(BaseModel):
    id: int
    name: str
    sortKey: int
    campusId: Optional[int] = None
    inMenu: bool
    fileAccessWithoutPermission: bool
    nameTranslated: str

    def __repr__(self) -> str:
        return f"<WikiCategory: {self.name} [{self.id}]>"


class WikiPermission(BaseModel):
    canEdit: bool


class WikiPage(BaseModel):
    identifier: str
    wikiCategory: WikiCategory
    title: str
    version: int
    text: Optional[str] = None
    onStartpage: EmptyStrToFalse[bool]
    redirectTo: Optional[str] = None
    permissions: WikiPermission
    isMarkdown: bool

    # TODO: meta

    def __repr__(self) -> str:
        return f"<WikiPage: {self.title} [{self.identifier}]>"


class WikiSearchResult(BaseModel):
    title: str
    domainType: str
    domainIdentifier: str
    apiUrl: str
    frontendUrl: str
    imageUrl: Optional[str] = None
    preview: str

    def __repr__(self) -> str:
        return f"<WikiSearchResult: {self.title} ({self.preview})>"



from datetime import date, datetime
from typing import List, Optional, Union

from pydantic import BaseModel, field_serializer  # type: ignore

from . import EmptyStrToNone


class CalendarMeta(BaseModel):
    modifiedDate: Union[date, datetime]
    modifiedPid: int


class Calendar(BaseModel):
    campusId: Optional[int] = None
    color: str
    eventTemplateId: Optional[int] = None
    iCalSourceUrl: Optional[str] = None
    id: int
    isPrivate: bool
    isPublic: bool
    meta: Optional[CalendarMeta] = None
    name: str
    nameTranslated: str
    randomUrl: str
    sortKey: int


class Address(BaseModel):
    meetingAt: Optional[str] = None
    street: Optional[str] = None
    addition: Optional[str] = None
    district: Optional[str] = None
    zip: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None
    latitude: Optional[EmptyStrToNone[float]] = None
    longitude: Optional[EmptyStrToNone[float]] = None


class AppointmentAddition(BaseModel):
    date: Union[date, datetime]
    isRepeated: Optional[bool] = False


class AppointmentException(BaseModel):
    date: Union[date, datetime]
    id: int
    meta: CalendarMeta


class PersonWithOnlyId(BaseModel):
    id: int


class AppointmentMeta(BaseModel):
    createdDate: Union[date, datetime]
    createdPerson: PersonWithOnlyId
    modifiedDate: Union[date, datetime]
    modifiedPerson: PersonWithOnlyId


class Appointment(BaseModel):
    additions: List[AppointmentAddition] = []
    address: Optional[Address] = None
    allDay: bool = False
    calendar: Optional[Calendar] = None
    caption: str
    endDate: Union[date, datetime]
    exceptions: List[AppointmentException] = []
    id: Optional[Union[int, str]] = None
    image: Optional[str] = None
    information: Optional[str] = None
    isInternal: bool = True
    link: Optional[str] = None
    meta: Optional[AppointmentMeta] = None
    note: Optional[str] = None
    onBehalfOfPid: Optional[int] = None
    startDate: Union[date, datetime]
    repeatFrequency: Optional[int] = None
    repeatId: int = 0
    repeatOption: Optional[int] = None
    repeatUntil: Optional[str] = None
    version: int = 0

    @field_serializer("startDate")
    def serialize_startDate(self, dt: datetime, _info):
        return dt.strftime("%Y-%m-%dT%H:%M:%SZ")

    @field_serializer("endDate")
    def serialize_endDate(self, dt: datetime, _info):
        return dt.strftime("%Y-%m-%dT%H:%M:%SZ")



"""
Service
=======

CRUD methods for services & service groups

TODO:

    - servicegroups

"""

from typing import Any, List

from .models.service import Service, ServiceGroup


class Services:
    def __init__(self, ct: Any) -> None:
        self.__ct = ct

    def list(self) -> List[Service]:
        """List all services.

        :returns: A list of all services
        :rtype: List[Service]
        """

        res = self.__ct.make_request("services")

        services = []
        if res and "data" in res:
            for item in res["data"]:
                services.append(Service(**item))

        return services

    def groups(self) -> List[ServiceGroup]:
        """List all service groups.

        :returns: A list of all service groups
        :rtype: List[ServiceGroup]
        """

        res = self.__ct.make_request("services")

        sgroups = []
        if res and "data" in res:
            for item in res["data"]:
                sgroups.append(ServiceGroup(**item))

        return sgroups



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class Service(BaseModel):
    id: int
    name: str
    serviceGroupId: int
    commentOnConfirmation: bool
    sortKey: int
    allowDecline: bool
    allowExchange: bool
    comment: str
    standard: bool
    hidePersonName: bool
    sendReminderMails: bool
    sendServiceRequestMails: Optional[bool] = None
    allowControlLiveAgenda: bool
    groupIds: Optional[str] = None
    tagIds: Optional[str] = None
    calTextTemplate: str
    allowChat: bool

    def __repr__(self) -> str:
        return f"<Service: {self.name} [{self.id}]>"


class ServiceGroup(BaseModel):
    id: int
    name: str
    sortKey: int
    viewAll: Optional[bool] = None
    campusId: Optional[int] = None
    onlyVisibleInCampusFilter: Optional[bool] = None

    def __repr__(self) -> str:
        return f"<ServiceGroup: {self.name} [{self.id}]>"


class ServiceRequest(BaseModel):
    id: int
    personId: int
    name: str
    serviceId: int
    agreed: bool
    isValid: bool
    requestedDate: datetime
    requesterPersonId: int
    comment: str
    counter: int

    # TODO: person
    # TODO: requesterPerson

    def __repr__(self) -> str:
        return f"<ServiceRequest: {self.name} {self.serviceId} [{self.id}]>"



from typing import Any, Generator, Generic, Optional, TypeVar

PydanticField = TypeVar("PydanticField")


class EmptyStrToNone(Generic[PydanticField]):
    @classmethod
    def __get_validators__(cls) -> Generator:
        yield cls.validate

    @classmethod
    def validate(cls, v: PydanticField, field: Any) -> Optional[PydanticField]:
        if v == "":
            return None
        return v


class EmptyStrToFalse(Generic[PydanticField]):
    @classmethod
    def __get_validators__(cls) -> Generator:
        yield cls.validate

    @classmethod
    def validate(cls, v: PydanticField, field: Any) -> bool:
        if v == "" or not v:
            return False
        return True
